from django.db import models
from .country import Country


class ShippingTag(models.Model):
    """
    Shipping information about a product or multiple products used to determine the shipping options avaliable for the final order.
    """

    class Meta:
        unique_together = ("category", "order")
        ordering = ["category", "order"]

    name = models.CharField(max_length=200)
    category = models.CharField(max_length=200)
    order = models.PositiveSmallIntegerField(
        default=0, help_text="Order of tag for the given category."
    )

    def __str__(self):
        return f"{self.category}: {self.name}"



from functools import partial
from secrets import token_hex

from django.conf import settings
from django.db import models
from django.apps import apps


class BaseCart(models.Model):
    class Meta:
        abstract = True
        indexes = (models.Index(fields=["cart_token"]),)

    # Used as a handle to retrieve the users cart from cookies or other
    cart_token = models.CharField(max_length=64, unique=True, editable=False,)

    checkout_details = models.ForeignKey(
        settings.SHOP_CHECKOUT_DETAILS_MODEL,
        null=True,
        related_name="+",
        on_delete=models.PROTECT,
    )

    def __str__(self):
        if not self.is_persisted:
            return f"Cart (Unpersisted)"

        return f"Cart ({self.cart_token[:8]})"

    def __len__(self):
        return self.items.count()  # pylint: disable=no-member

    @property
    def is_persisted(self):
        """
        Flag indicating if the cart has been stored in the db.
        """

        return bool(self.cart_token)

    @property
    def empty(self):
        return len(self) == 0

    @property
    def subtotal(self):
        total = 0
        for item in self.items.all():  # pylint: disable=no-member
            total += item.product.price * item.quantity
        return total

    def save(self, *args, **kwargs):

        if not self.cart_token:
            self.cart_token = token_hex(32)

        super().save(*args, **kwargs)

    def add_item(self, product, *, quantity=1):
        """
        Add item to cart. A single quantity will be added unless specified otherwise.
        """

        # Ensure we are using a persisted cart
        self.save()
        self.refresh_from_db()

        CartItem = apps.get_model(settings.SHOP_CART_ITEM_MODEL)

        item, created = CartItem.objects.get_or_create(
            cart=self, product=product, defaults={"quantity": quantity}
        )

        if not created:
            item.quantity += quantity
            item.save()

    def remove_item(self, product, *, quantity=None):
        """
        Remove item from cart. If quantity is None, the full quantity of the given item will be removed.
        """

        # Ensure we are using a persisted cart
        self.save()
        self.refresh_from_db()

        CartItem = apps.get_model(settings.SHOP_CART_ITEM_MODEL)
        item = CartItem.objects.get(cart=self, product=product)

        if quantity is None or quantity == item.quantity:
            CartItem.objects.filter(pk=item.pk).delete()

        else:
            item.quantity -= quantity
            item.save()

    def update_item(self, product, *, quantity):
        """
        Update item in cart to a specific quantity. A quantity of 0 will remove the item.
        """

        # Ensure we are using a persisted cart
        self.save()
        self.refresh_from_db()

        CartItem = apps.get_model(settings.SHOP_CART_ITEM_MODEL)

        item, created = CartItem.objects.get_or_create(
            cart=self, product=product, defaults={"quantity": quantity}
        )

        if not created:
            if quantity == 0:
                CartItem.objects.filter(pk=item.pk).delete()

            else:
                item.quantity = quantity
                item.save()



from typing import Optional
from django.db import models

from ..concrete.address import Address
from ..concrete.shipping_option import ShippingOption


class BaseCheckoutDetails(models.Model):
    class Meta:
        abstract = True

    customer_name = models.CharField(max_length=100)
    customer_company = models.CharField(max_length=100, blank=True)
    customer_email = models.EmailField()
    customer_phone = models.CharField(max_length=100)
    customer_coupon = models.CharField(max_length=100, blank=True)

    shipping_selection = models.ForeignKey(
        ShippingOption, null=True, on_delete=models.PROTECT
    )

    shipping_address = models.ForeignKey(
        Address, related_name="+", on_delete=models.PROTECT
    )

    billing_address_same_as_shipping = models.BooleanField(default=True)

    # Private billing details fields to be accessed via setter + getter
    #   These fields will default to the customer/shipping information depending on
    #   the state of the 'billing_address_same_as_shipping' flag.
    _billing_name = models.CharField(max_length=100, blank=True)
    _billing_company = models.CharField(max_length=100, blank=True)
    _billing_address = models.ForeignKey(
        Address, null=True, related_name="+", on_delete=models.PROTECT
    )

    @property
    def billing_name(self):
        if self.billing_address_same_as_shipping:
            return self.customer_name
        return self._billing_name

    @billing_name.setter
    def billing_name(self, name):
        self._billing_name = name

    @property
    def billing_company(self):
        if self.billing_address_same_as_shipping:
            return self.customer_company
        return self._billing_company

    @billing_company.setter
    def billing_company(self, company):
        self._billing_company = company

    @property
    def billing_address(self):
        if self.billing_address_same_as_shipping:
            return self.shipping_address
        return self._billing_address

    @billing_address.setter
    def billing_address(self, address):
        self._billing_address = address



from django.db import models
from .country import Country
from .shipping_tag import ShippingTag


class ShippingOption(models.Model):
    name = models.CharField(max_length=200)
    price = models.DecimalField(max_digits=5, decimal_places=2)

    # Look up criteria
    country = models.ForeignKey(Country, null=True, on_delete=models.CASCADE)
    shipping_tag = models.ForeignKey(ShippingTag, on_delete=models.CASCADE)

    def __str__(self):
        return f"{self.name} (${self.price})"



from django.db import models
from ..concrete.image import Image
from ..concrete.shipping_tag import ShippingTag


class BaseProduct(models.Model):
    class Meta:
        abstract = True
        ordering = ["-order_priority"]

    # Identifiers
    handle = models.CharField(
        max_length=200,
        unique=True,
        help_text="Unique idenitfier used select specific product.",
    )

    # Descriptive info
    name = models.CharField(max_length=200)
    description = models.TextField()

    # Product images
    images = models.ManyToManyField(Image)

    # Pricing
    price = models.DecimalField(max_digits=7, decimal_places=2)
    shipping_tag = models.ForeignKey(ShippingTag, on_delete=models.PROTECT)

    # Availablity
    available = models.BooleanField(default=True)
    back_in_stock = models.DateField(null=True, blank=True)

    # Admin only
    order_priority = models.PositiveSmallIntegerField(
        default=1000, help_text="Hint on how the product should be displayed in lists."
    )

    def __str__(self):
        return f"{self.name} ({self.handle})"

    @property
    def first_image(self):
        return self.images.all().first()



from django.db import models

from .country import Country


class Address(models.Model):
    street = models.CharField(max_length=100)
    city = models.CharField(max_length=100)
    state = models.CharField(max_length=100)
    postcode = models.CharField(max_length=20)

    country = models.ForeignKey(Country, on_delete=models.PROTECT)



from django.apps import apps
from django.conf import settings

from .concrete.image import Image

from .concrete.country import Country
from .concrete.address import Address

from .concrete.shipping_tag import ShippingTag
from .concrete.shipping_option import ShippingOption

from .abstract.base_product import BaseProduct
from .abstract.base_checkout_details import BaseCheckoutDetails

from .abstract.base_order_item import BaseOrderItem
from .abstract.base_order import BaseOrder

from .abstract.base_cart_item import BaseCartItem
from .abstract.base_cart import BaseCart




from django.apps import apps
from django.conf import settings

Cart = apps.get_model(settings.SHOP_CART_MODEL)


class CartMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):

        # Request
        cart_token = request.COOKIES.get("cart_token")
        request.cart = None

        if cart_token:
            request.cart = Cart.objects.filter(cart_token=cart_token).first()

        if request.cart is None:
            request.cart = Cart()

        # Response
        response = self.get_response(request)

        if request.cart.is_persisted:
            response.set_cookie("cart_token", request.cart.cart_token)

        return response



# Generated by Django 3.0.2 on 2020-01-22 17:27
# type: ignore

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Country",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=200)),
            ],
        ),
        migrations.CreateModel(
            name="Image",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "handle",
                    models.CharField(db_index=True, max_length=200, unique=True),
                ),
                ("image", models.ImageField(upload_to="images/")),
            ],
        ),
        migrations.CreateModel(
            name="ShippingTag",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=200)),
                ("category", models.CharField(max_length=200)),
                (
                    "order",
                    models.PositiveSmallIntegerField(
                        default=0, help_text="Order of tag for the given category."
                    ),
                ),
            ],
            options={
                "ordering": ["category", "order"],
                "unique_together": {("category", "order")},
            },
        ),
        migrations.CreateModel(
            name="ShippingOption",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=200)),
                ("price", models.DecimalField(decimal_places=2, max_digits=5)),
                (
                    "country",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="django_base_shop.Country",
                    ),
                ),
                (
                    "shipping_tag",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="django_base_shop.ShippingTag",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Address",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("street", models.CharField(max_length=100)),
                ("city", models.CharField(max_length=100)),
                ("state", models.CharField(max_length=100)),
                ("postcode", models.CharField(max_length=20)),
                (
                    "country",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="django_base_shop.Country",
                    ),
                ),
            ],
        ),
    ]



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="django-base-shop",
    version="0.0.11",
    author="Blake Lockley",
    author_email="bwlockley@gmail.com",
    description="A basic, extendable data model for ecommerce shops using Django",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/blakelockley/django-base-shop",
    packages=setuptools.find_packages(exclude=["test"]),
    install_requires=["django >= 2.1", "pillow == 7.0.0"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.6.5",
)
